@@ -3,21 +3,18 @@
 
 __author__ = 'Anindita Dutta, Ahmet Bakan, Cihan Kaya'
 
+from prody.dynamics.analysis import calcFracDimension
 import re
 from numbers import Integral
 
 import numpy as np
-import os
 from os.path import join, isfile
 from io import BytesIO
-import zlib
 
 from prody import LOGGER, PY3K
 from prody.utilities import makePath, openURL, gunzip, openFile, dictElement
 from prody.utilities import relpath
 from prody.proteins import parsePDB
-from prody.sequence import parseMSA, refineMSA, MSA
-from .uniprot import queryUniprot
 
 if PY3K:
     import urllib.parse as urllib
@@ -26,7 +23,8 @@ else:
     import urllib
     import urllib2
 
-__all__ = ['searchPfam', 'fetchPfamMSA', 'searchUniprotID', 'parsePfamPDBs']
+
+__all__ = ['searchPfam', 'fetchPfamMSA', 'parsePfamPDBs']
 
 FASTA = 'fasta'
 SELEX = 'selex'
@@ -60,6 +58,8 @@ def searchPfam(query, **kwargs):
     chain identifier.  UniProt ID of the specified chain, or the first
     protein chain will be used for searching the Pfam database."""
 
+    import requests
+
     if isfile(query):
         from prody.sequence import MSAFile
         try:
@@ -205,14 +205,19 @@ def searchPfam(query, **kwargs):
 
     LOGGER.debug('Retrieving Pfam search results: ' + url)
     xml = None
+    sleep = 2
     while LOGGER.timing('_pfam') < timeout:
         try:
-            xml = openURL(url, timeout=timeout).read()
+            # xml = openURL(url, timeout=timeout).read()
+            xml = requests.get(url, verify=False).content
         except Exception:
             pass
         else:
             if xml not in ['PEND','RUN']:
                 break
+        
+        sleep = 20 if int(sleep * 1.5) >= 20 else int(sleep * 1.5)
+        LOGGER.sleep(int(sleep), '. Trying to reconnect...')
 
     if not xml:
         raise IOError('Pfam search timed out or failed to parse results '
@@ -226,6 +231,7 @@ def searchPfam(query, **kwargs):
     elif xml.find(b'No valid UniProt accession or ID') > 0:
         try:
             url = prefix + 'protein/' + accession + '?output=xml'
+            LOGGER.debug('Retrieving Pfam search results: ' + url)
             xml = openURL(url, timeout=timeout).read()
         except:
             try:
@@ -234,6 +240,19 @@ def searchPfam(query, **kwargs):
                 return searchPfam(ag_seq)
             except:
                 raise ValueError('No valid UniProt accession or ID for: ' + seq)
+        
+        if xml.find(b'No valid UniProt accession or ID') > 0:
+            try:
+                url = 'https://uniprot.org/uniprot/' + accession + '.xml'
+                xml = openURL(url, timeout=timeout).read()
+                root = ET.XML(xml)
+                accession = root[0][0].text
+
+                url = prefix + 'protein/' + accession + '?output=xml'
+                LOGGER.debug('Retrieving Pfam search results: ' + url)
+                xml = openURL(url, timeout=timeout).read()                
+            except:
+                raise ValueError('No valid UniProt accession or ID for: ' + seq)
 
     try:
         root = ET.XML(xml)
@@ -281,71 +300,6 @@ def searchPfam(query, **kwargs):
         LOGGER.info(query + ' did not match any Pfam families.')
     return matches
 
-def searchUniprotID(query, search_b=False, skip_a=False, **kwargs):
-    """Returns Pfam search results in a dictionary.  Matching Pfam accession
-    as keys will map to evalue, alignment start and end residue positions.
-
-    :arg query: UniProt ID, PDB identifier, protein sequence, or a 
-        sequence file. Sequence queries must not contain gaps and 
-        must be at least 16 characters long
-    :type query: str
-
-    :arg search_b: search Pfam-B families when **True**
-    :type search_b: bool
-
-    :arg skip_a: do not search Pfam-A families when **True**
-    :type skip_a: bool
-
-    :arg ga: use gathering threshold when **True**
-    :type ga: bool
-
-    :arg evalue: user specified e-value cutoff, must be smaller than 10.0
-    :type evalue: float
-
-    :arg timeout: timeout for blocking connection attempt in seconds, default
-        is 60
-    :type timeout: int
-
-    *query* can also be a PDB identifier, e.g. ``'1mkp'`` or ``'1mkpA'`` with
-    chain identifier.  UniProt ID of the specified chain, or the first
-    protein chain will be used for searching the Pfam database."""
-
-    query = str(query)
-    seq = ''.join(query.split())
-
-    import xml.etree.cElementTree as ET
-    LOGGER.timeit('_pfam')
-    timeout = int(kwargs.get('timeout', 60))
-    url = prefix + 'protein/' + seq + '?output=xml'
-
-    LOGGER.debug('Retrieving Pfam search results: ' + url)
-    xml = None
-    while LOGGER.timing('_pfam') < timeout:
-        try:
-            xml = openURL(url, timeout=timeout).read()
-        except Exception:
-            pass
-        else:
-            if xml:
-                break
-
-    if not xml:
-        raise IOError('Pfam search timed out or failed to parse results '
-                      'XML, check URL: ' + url)
-    else:
-        LOGGER.report('Pfam search completed in %.2fs.', '_pfam')
-
-    if xml.find(b'There was a system error on your last request.') > 0:
-        LOGGER.warn('No Pfam matches found for: ' + seq)
-        return None
-
-    try:
-        root = ET.XML(xml)
-    except Exception as err:
-        raise ValueError('failed to parse results XML, check URL: ' + url)
-
-    result = root[0].get('id')
-    return result
 
 def fetchPfamMSA(acc, alignment='full', compressed=False, **kwargs):
     """Returns a path to the downloaded Pfam MSA file.
@@ -381,13 +335,15 @@ def fetchPfamMSA(acc, alignment='full', compressed=False, **kwargs):
     :arg outname: out filename, default is input ``'acc_alignment.format'``
 
     :arg folder: output folder, default is ``'.'``"""
+    
+    import requests
 
-    url = prefix + 'family/acc?id=' + acc
-    handle = openURL(url, timeout=int(kwargs.get('timeout', 60)))
+    # url = prefix + 'family/acc?id=' + acc
+    # handle = openURL(url, timeout=int(kwargs.get('timeout', 60)))
     orig_acc = acc
-    acc = handle.readline().strip()
-    if PY3K:
-        acc = acc.decode()
+    # acc = handle.readline().strip()
+    # if PY3K:
+    #     acc = acc.decode()
     url_flag = False
 
     if not re.search('(?<=PF)[0-9]{5}$', acc):
@@ -440,7 +396,23 @@ def fetchPfamMSA(acc, alignment='full', compressed=False, **kwargs):
                    '&alnType=' + alignment + '&order=' + order[0] +
                    '&case=' + inserts[0] + '&gaps=' + gaps + '&download=1')
 
-    response = openURL(url, timeout=int(kwargs.get('timeout', 60)))
+    LOGGER.timeit('_pfam')
+    timeout = kwargs.get('timeout', 60)
+    response = None
+    sleep = 2
+    try_error = 3
+    while LOGGER.timing('_pfam') < timeout:
+        try:
+            response = requests.get(url, verify=False).content
+        except Exception:
+            pass
+        else:
+            break
+        
+        sleep = 20 if int(sleep * 1.5) >= 20 else int(sleep * 1.5)
+        LOGGER.sleep(int(sleep), '. Trying to reconnect...')
+
+    # response = openURL(url, timeout=int(kwargs.get('timeout', 60)))
     outname = kwargs.get('outname', None)
     if not outname:
         outname = orig_acc
@@ -452,14 +424,16 @@ def fetchPfamMSA(acc, alignment='full', compressed=False, **kwargs):
             f_out = open(filepath, 'wb')
         else:
             f_out = openFile(filepath, 'wb')
-        f_out.write(response.read())
+        # f_out.write(response.read())
+        f_out.write(response)
         f_out.close()
     else:
         if url_flag:
             gunzip(response.read(), filepath)
         else:
             with open(filepath, 'wb') as f_out:
-                f_out.write(response.read())
+                # f_out.write(response.read())
+                f_out.write(response)
 
     filepath = relpath(filepath)
     LOGGER.info('Pfam MSA for {0} is written as {1}.'
